<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\NovoChecklistService;

class NovoChecklistController extends FMCrud{
    protected $dataSourceName = 'fcm_checklist';
    protected $NovoChecklistService;
   
    public function __construct($crudService, $environment, NovoChecklistService $NovoChecklistService) {
        parent::__construct($crudService, $environment);
        $this->NovoChecklistService = $NovoChecklistService;
    }

    public function getLocal(DTO\Request $request, DTO\Response $response) {
        $novoFiltro = $this->NovoChecklistService->buscaNomeLocais();
        $dtsVal = new DataSet('local', $novoFiltro);
        $response->addDataSet($dtsVal);
    }
    
    public function getDadosCard(DTO\Request $request, DTO\Response $response){
        $params = $request->getParameters();
        $dadosLocal = $this->NovoChecklistService->buscaNomeLocal($params);
        $dtsVal1 =  new DataSet('dadosLocal', $dadosLocal);
        $response->addDataSet($dtsVal1);
    }
    
    public function buscaDadosCardPadrao(DTO\Request $request, DTO\Response $response){
        $dados =  $this->NovoChecklistService->buscaDadosCardPadrao();
        $dtsVal =  new DataSet('dados', $dados);
        $response->addDataSet($dtsVal);
    }
    
    public function buscaDadosAtividadeseCriterios(DTO\Request $request, DTO\Response $response){
        $params = $request->getParameters();
        $dados =  $this->NovoChecklistService->buscaDadosAtividadeseCriterios($params);
        $dtsVal =  new DataSet('dados', $dados);
        $response->addDataSet($dtsVal);
    }
    
    public function buscaDadosCardAtividades(DTO\Request $request, DTO\Response $response){
        $dados =  $this->NovoChecklistService->buscaDadosCardAtividades();
        $dtsVal =  new DataSet('dados', $dados);
        $response->addDataSet($dtsVal);
    }
    
    public function insertNovaAtividadeBase(DTO\Request $request, DTO\Response $response){
        
        $parametro = $request->getParameters();
        // debugApi($parametro);
        
        $nratividade      = $parametro['novaAtividade']['NRATIVIDADE'];
        $nmatividade = $parametro['novaAtividade']['NMATIVIDADE'];
        $nrtarefa = $parametro['novaAtividade']['NRTAREFA'];
        $nmtarefa = $parametro['novaAtividade']['NMTAREFA'];
        $nratividade = $parametro['novaAtividade']['NRATIVIDADE'];
        $dspopservico = $parametro['novaAtividade']['DSPOPSERVICO'];
        $nrcriteriotarefa = $parametro['novaAtividade']['NRCRITERIOTAREFA'];
        $dscriteriotarefa = $parametro['novaAtividade']['DSCRITERIOTAREFA'];
        $idtipocriterio = $parametro['novaAtividade']['IDTIPOCRITERIO'];
        $nrtagstarefa = $parametro['novaAtividade']['NRTAGSTAREFA'];
        $dstagstarefa = $parametro['novaAtividade']['DSTAGSTAREFA'];
        $idativo = $parametro['novaAtividade']['IDATIVO'];
        $nrorg = $parametro['novaAtividade']['NRORG'];
    
        $var =  $this->NovoChecklistService->insertNovaAtividadeBase($nratividade, $nmatividade, $nrtarefa, $nmtarefa, $dspopservico, $nrcriteriotarefa, $dscriteriotarefa, $idtipocriterio, $nrtagstarefa, $dstagstarefa);
        $dtsVal =  new DataSet('novaAtividade', $var);
        $response->addDataSet($dtsVal);
    }
    
    public function filtroItens(DTO\Request $request, DTO\Response $response) {
        
        $novoFiltro =  $this->NovoChecklistService->filtroItens();
        $dtsVal =  new DataSet('filtro', $novoFiltro);
        $response->addDataSet($dtsVal);
        
    }

    public function filtroTags(DTO\Request $request, DTO\Response $response) {
            $parametro = $request->getParameters();
            // debugApi($parametro);
            $novoFiltro =  $this->NovoChecklistService->filtroTags($parametro);
            $dtsVal =  new DataSet('filtroTags', $novoFiltro);
            $response->addDataSet($dtsVal);
            
    }
    
    public function insertNovoChecklistBase(DTO\Request $request, DTO\Response $response) {
        
        $parametro = $request->getParameters();
        
        // Tabela - FCM_ATIVIDADECHK
        
        $nratividadechk = $parametro['arrayDados']['NRATIVIDADECHK'];
        $nratividade = $parametro['arrayDados']['NRATIVIDADE'];
        $nrchecklist = $parametro['arrayDados']['NRCHECKLIST'];
        $idativo = $parametro['arrayDados']['IDATIVO'];
        $nrorg = $parametro['arrayDados']['NRORG'];
        $dtinclusao = $parametro['arrayDados']['DTINCLUSAO'];
        
        // Tabela - FCM_TAREFACHK
        //Dados Iguais a table acima IDATIVO, NRORG, DTINCLUSAO, NRCHECKLIST, NRATIVIDADECHK
        $nrtarefachk = $parametro['arrayDados']['NRTAREFACHK'];
        $nrtarefa = $parametro['arrayDados']['NRTAREFA'];
        
        // Tabela - FCM_CRITERIOTAREFACHK
        //Dados Iguais a table acima IDATIVO, NRORG, DTINCLUSAO, NRTAREFACHK,
        $nrcriteriotarefachk = $parametro['arrayDados']['NRCRITERIOTAREFACHK'];
        $nrcriteriotarefa = $parametro['arrayDados']['NRCRITERIOTAREFA'];
        
        // Tabela - FCM_TAGSCHKTAREFA
        //Dados Iguais a table acima IDATIVO, NRORG, DTINCLUSAO, NRTAREFACHK,
        $nrtagschktarefa = $parametro['arrayDados']['NRTAGSCHKTAREFA'];
        $nrtagstarefa = $parametro['arrayDados']['NRTAGSTAREFA'];
        
        $var = $this->NovoChecklistService->insertNovoChecklistBase(/*Passar Parametros vindo do front-> controler-> Service (Verificar com o Arthur)*/);
        $dtsVal =  new DataSet('arrayDados', $var);
        $response->addDataSet($dtsVal);
    }
    
    public function getGrupo(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $novoFiltro = $this->NovoChecklistService->criaGruposTags($parametro);
        $dtsVal = new DataSet('grupos', $novoFiltro);
        $response->addDataSet($dtsVal);
        
    }
    
    public function buscaDadosCriteriosNaoConformidades(DTO\Request $request, DTO\Response $response) {
        $parametro1 = $request->getParameters();
        $parametro2 = $request->getParameters();
        $novoFiltro = $this->NovoChecklistService->buscaDadosCriteriosNaoConformidades($parametro1, $parametro2);
        $dtsVal = new DataSet('dados', $novoFiltro);
        $response->addDataSet($dtsVal);
    }
    // controller responsavel por passar o parametro do NRchecklist do Front para o BackEnd
    public function getDadosEditCard(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $dadosEditCard = $this->NovoChecklistService->getDadosEditCard($parametro['nr']);
        $dtsVal = new DataSet('dados', $dadosEditCard);
        $response->addDataSet($dtsVal);
    }
    
    public function addNovaTarefaCardAtividades(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $dados = $parametro['DADOS'];
        $nrorg = $parametro['NRORG'];
        $this->NovoChecklistService->insertDadostarefaModal($dados,$nrorg);
        $dtsVal = new DataSet('dados', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function getDadosCopiaCard(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $checklist = $parametro['NRCHECKLIST'];
        $nrorg = $parametro['NRORG'];
        $this->NovoChecklistService->getDadosCopiaCard($checklist,$nrorg);
        $dtsVal = new DataSet('selectDadosCard', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function addNovaTarefa(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        // debugApi([$parametro]);
        $dados = $parametro['DADOS'];
        $nrorg = $parametro['NRORG'];
        $this->NovoChecklistService->insertDadosNovatarefa($dados,$nrorg);
        $dtsVal = new DataSet('dados', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    //----------------------------------------------------------------//
    
    public function insertChecklist(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $dados = $parametro['DADOS'];
        $nrorg = $parametro['NRORG'];
        $this->NovoChecklistService->insertChecklist($dados, $nrorg);
        $dtsVal =  new DataSet('resposta', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function validacao(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $dadosEditCard = $this->NovoChecklistService->validacaoChecklist($parametro['nr']);
        $dtsVal = new DataSet('validacao', $dadosEditCard);
        $response->addDataSet($dtsVal);
    }

    public function validacaoTarefas(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $parametroValidacaoTarefa = $this->NovoChecklistService->validacaoTarefa($parametro);
        $retornoTarefa = new DataSet('validacaoTarefa', $parametroValidacaoTarefa);
        $response->addDataSet($retornoTarefa);

    }
    
    public function validacaoCriterio(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $parametroValidacaoCriterio = $this->NovoChecklistService->validacaoCriterio($parametro);
        $retornoCriterio = new DataSet('validacaoCriterio', $parametroValidacaoCriterio);
        $response->addDataSet($retornoCriterio);
    }
    
    public function validacaoTag(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $parametroValidacaoTag = $this->NovoChecklistService->validacaoTag($parametro);
        $retornoTag = new DataSet('validacaoTag', $parametroValidacaoTag);
        $response->addDataSet($retornoTag);
    }
    
    public function updateChecklist(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $dados = $parametro['DADOS'];
        $nrorg = $parametro['NRORG'];
        $nr = $parametro['NRCHECKLIST'];
        $this->NovoChecklistService->updateChecklist($dados, $nrorg, $nr);
        $dtsVal =  new DataSet('resposta', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function deletaTarefaAtividade(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $nrCriterioTarefa = $parametro['tarefas']["NRCRITERIOTAREFA"];
        $nrTagsTarefa = $parametro['tarefas']["NRTAGSTAREFA"];
        $nrTarefa = $parametro['tarefas']["NRTAREFA"];
        $this->NovoChecklistService->deletaTarefaAtividade($nrCriterioTarefa, $nrTagsTarefa,$nrTarefa);
        $dtsVal = new DataSet('dados', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function deletaAtividade(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $nrAtividade = $parametro['tarefas']["NRATIVIDADE"];
        $this->NovoChecklistService->deletaAtividade($nrAtividade);
        $dtsVal = new DataSet('dados', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function deleteChecklist(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $nr = $parametro['NRCHECKLIST'];
        $this->NovoChecklistService->deleteChecklist($nr);
        $dtsVal = new DataSet('dados', array('retorno' => 'OK'));
        $response->addDataSet($dtsVal);
    }
    
    public function changeColorActivity(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $retorno = $this->NovoChecklistService->changeColorActivity($parametro);
        $dtsVal = new DataSet('dados', [$retorno]);
        $response->addDataSet($dtsVal);
    }
}
?>