import {
  Button, Col, Form, IModal, Modal, ModalService, Select, TextInput, AlertService,
} from '@zeedhi/common';
import { IDictionary, Http, Metadata } from '@zeedhi/core';

export class ParametrosSensorController {
  public mapAlertModal: IDictionary = {};

  public SensorModal!: Modal;

  public Sensors: IDictionary = {};

  public Clients: Array<IDictionary> = [];

  public Locais: Array<IDictionary> = [];

  public TypeSensors: Array<IDictionary> = [];

  public DistanceMode: Array<IDictionary> = [];

  public disabledSensorFields: boolean = true;

  public CardCounter: number = 0;

  public listSensor({ component }: { component: Col }) {
    Http.get('/ParamLeitContPessoas')
      .then((response: IDictionary) => {
        const sensors = response.data.dataset.Sensores;
        sensors.map((sensor: IDictionary, index: number) => {
          const card = this.newCard(index.toString(), sensor.IDMACDEVICE, sensor.CDCLIENTE, sensor.NRPARAMLEITCONTPESSOAS.toString());
          component.children.push(card);
          this.CardCounter += 1;
          return card;
        });
      });
  }

  public listLocais({ component }: { component: Select }) {
    const { value } = component;
    Http.get(`/Locals?CDCLIENTE=${value}`)
      .then((response: IDictionary) => {
        const selectLocal: Select = Metadata.getInstance('NRLOCALCLIENTE');
        this.Locais = response.data.dataset.Locals;
        selectLocal.datasource.updateData(this.Locais);
      });
  }

  public listClients({ component }: { component: Select}) {
    Http.get('/Clients/simpleView')
      .then((response: IDictionary) => {
        this.Clients = response.data.dataset.Clientes;
        component.datasource.updateData(this.Clients);
      });
  }

  public listTypeSensors({ component }: { component: Select }) {
    const fixed = [
      {
        id: 1,
        name: 'Infravermelho',
      },
      {
        id: 2,
        name: 'Distância',
      },
      {
        id: 3,
        name: 'Câmera',
      },
    ];
    component.datasource.updateData(fixed);
  }

  public listDistanceMode({ component }: { component: Select }) {
    const fixedDistances = [
      {
        id: 1,
        name: 'Curta',
      },
      {
        id: 2,
        name: 'Longa',
      },
    ];
    component.datasource.updateData(fixedDistances);
  }

  public listAlertTypes({ component } : { component: Select}) {
    const fixedAlertTypes = [
      {
        id: 1,
        name: 'Email',
      },
    ];

    component.datasource.updateData(fixedAlertTypes);
  }

  public newCard(v1: string, macAddress: string, client: string, idSensor: string) {
    return {
      name: `col2-${v1}`,
      component: 'ZdCol',
      cols: 12,
      children: [
        {
          name: `card-2-${v1}`,
          component: 'ZdCard',
          hover: true,
          elevation: 5,
          children: [
            {
              name: `insideRow2-${v1}`,
              component: 'ZdRow',
              children: [
                {
                  name: `insideCol-2-${v1}-1`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: 'RowCient',
                      component: 'ZdRow',
                      align: 'center',
                      children: [
                        {
                          name: `icon-2-${v1}-1`,
                          component: 'ZdButton',
                          iconName: 'mdi-account',
                          icon: true,
                          small: true,
                        },
                        {
                          name: `text-2-${v1}-2`,
                          component: 'ZdText',
                          cssClass: 'zd-pl-3',
                          text: client,
                        },
                      ],
                    },
                  ],
                },
                {
                  name: `insideCol-2-${v1}-3`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: 'RowMACAdress',
                      component: 'ZdRow',
                      align: 'center',
                      children: [
                        {
                          name: `icon2-${v1}-3`,
                          component: 'ZdButton',
                          iconName: 'mdi-desktop-mac',
                          icon: true,
                          small: true,
                        },
                        {
                          name: `text-2-${v1}-4`,
                          component: 'ZdText',
                          cssClass: 'zd-pl-3',
                          text: macAddress,
                        },
                      ],
                    },
                  ],
                },
                {
                  name: `insideCol-2-${v1}-5`,
                  component: 'ZdCol',
                  cols: 4,
                  align: 'right',
                  children: [
                    {
                      name: `${idSensor}`,
                      component: 'ZdButton',
                      icon: true,
                      iconName: 'mdi-bell',
                      small: true,
                      events: {
                        onCreated: this.createAlertModal.bind(this),
                        onBeforeDestroy: this.destroyAlertModal.bind(this),
                        click: this.openAlertModal.bind(this),
                      },
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public getSensorExclusiveFields() {
    const elements = [];
    const distMin: TextInput = Metadata.getInstance('VRDISTANCIAMINLEIT');
    const distMax: TextInput = Metadata.getInstance('VRDISTANCIAMAXLEIT');
    const distThresh: TextInput = Metadata.getInstance('VRDISTANCIAMEDLEIT');
    const rowsOfSpad: TextInput = Metadata.getInstance('QTMATRIZLEITURA');
    const timmingBudget: TextInput = Metadata.getInstance('QTTEMPRECONHECIMENTO');
    const distanceMode: Select = Metadata.getInstance('IDMODODISTANCIA');
    elements.push(distMin);
    elements.push(distMax);
    elements.push(distThresh);
    elements.push(rowsOfSpad);
    elements.push(timmingBudget);
    elements.push(distanceMode);
    return elements;
  }

  public getGeneralFields() {
    const elements = [];
    const codCliente: Select = Metadata.getInstance('CDCLIENTE');
    const localCliente: Select = Metadata.getInstance('NRLOCALCLIENTE');
    const idTipoSensor: Select = Metadata.getInstance('IDTIPOSENSOR');
    const macAddress: TextInput = Metadata.getInstance('IDMACDEVICE');
    const idSensor: TextInput = Metadata.getInstance('IDSENSOR');
    const quantPeopleAlert: TextInput = Metadata.getInstance('NRQTPESSOASALERT');
    const hourInit: TextInput = Metadata.getInstance('HRINICIODIA');
    const hourEnd: TextInput = Metadata.getInstance('HRFINALDIA');
    const intervalRequests: TextInput = Metadata.getInstance('QTTEMPOREPETLEIT');
    elements.push(codCliente);
    elements.push(localCliente);
    elements.push(idTipoSensor);
    elements.push(macAddress);
    elements.push(idSensor);
    elements.push(quantPeopleAlert);
    elements.push(hourInit);
    elements.push(hourEnd);
    elements.push(intervalRequests);
    return elements;
  }

  public changeDisableFields({ component }: { component: Select }) {
    const { value } = component;
    // apenas inverte o valor do atributo disabled no componente
    if ((value === 2 && this.disabledSensorFields) || (value !== 2 && !this.disabledSensorFields)) {
      const fields = this.getSensorExclusiveFields();
      this.disabledSensorFields = !this.disabledSensorFields;
      fields.map((item) => {
        item.disabled = this.disabledSensorFields;
        return 0;
      });
    }
  }

  public createSensorModal() {
    const sensorModal: IModal = {
      name: 'sensorModal',
      persistent: false,
      fullscreen: false,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'modalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cols: 12,
              children: [
                {
                  name: 'titleText',
                  component: 'ZdText',
                  text: 'Novo sensor',
                  cssClass: 'text-center',
                  style: {
                    color: '#0077B6',
                  },
                },
              ],
            },
          ],
        },
        {
          name: 'sensorForm',
          component: 'ZdForm',
          children: [
            {
              name: 'row4',
              component: 'ZdRow',
              children: [
                {
                  name: 'col-4-1',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'CDCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        data: [],
                      },
                      dataText: 'NMRAZSOCCLIE',
                      dataValue: 'CDCLIENTE',
                      label: 'Cliente',
                      events: {
                        onCreated: this.listClients.bind(this),
                        change: this.listLocais.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-2',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRLOCALCLIENTE',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        data: [],
                      },
                      dataText: 'NMLOCALCLIENTE',
                      dataValue: 'NRLOCALCLIENTE',
                      label: 'Local',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-3',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDTIPOSENSOR',
                      component: 'ZdSelect',
                      autocomplete: false,
                      datasource: {
                        data: [],
                      },
                      dataValue: 'id',
                      dataText: 'name',
                      label: 'Tipo de sensor',
                      events: {
                        onCreated: this.listTypeSensors.bind(this),
                        change: this.changeDisableFields.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-4',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMACDEVICE',
                      component: 'ZdTextInput',
                      label: 'MAC Address do Microcontrolador',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-5',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDSENSOR',
                      component: 'ZdTextInput',
                      label: 'ID Sensor',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-6',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'NRQTPESSOASALERT',
                      component: 'ZdTextInput',
                      label: 'Número de pessoas para enviar o alerta',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-14',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRINICIODIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de inicio da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-15',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'HRFINALDIA',
                      component: 'ZdTime',
                      timeFormat: '24hr',
                      label: 'Horário de fim da contagem',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-13',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'QTTEMPOREPETLEIT',
                      component: 'ZdTextInput',
                      label: 'Intervalo entre os registros (segundos)',
                      type: 'number',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-4-7',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMINLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância minima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-8',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMAXLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância máxima (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-9',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'VRDISTANCIAMEDLEIT',
                      component: 'ZdTextInput',
                      label: 'Distância limite (mm)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-10',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTMATRIZLEITURA',
                      component: 'ZdTextInput',
                      label: 'Numero de linhas da matriz',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-11',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'QTTEMPRECONHECIMENTO',
                      component: 'ZdTextInput',
                      label: 'Tempo de leitura (ms)',
                      type: 'number',
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
                {
                  name: 'col-4-12',
                  component: 'ZdCol',
                  cols: 6,
                  children: [
                    {
                      name: 'IDMODODISTANCIA',
                      component: 'ZdSelect',
                      autocomplete: false,
                      label: 'Modo de distância',
                      datasource: {
                        data: [],
                      },
                      dataText: 'name',
                      dataValue: 'id',
                      events: {
                        onCreated: this.listDistanceMode.bind(this),
                      },
                      disabled: this.disabledSensorFields,
                    },
                  ],
                },
              ],
            },
            {
              name: 'row5',
              component: 'ZdRow',
              children: [
                {
                  name: 'col5-1',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'sensorButton',
                      component: 'ZdButton',
                      label: 'Criar novo sensor',
                      events: {
                        click: this.insertSensor.bind(this),
                      },
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
    this.SensorModal = ModalService.create(sensorModal);
  }

  public destroySensorModal() {
    this.SensorModal.destroy();
  }

  public openSensorModal() {
    this.SensorModal.show();
  }

  public closeSensorModal() {
    this.SensorModal.hide();
  }

  public createAlertModal({ component }: { component: Button }) {
    const idSensor = component.name;
    const aModal: IModal = {
      name: `alertModal-${idSensor}`,
      component: 'ZdModal',
      persistent: false,
      fullscreen: false,
      style: {
        background: '#black',
      },
      children: [
        {
          name: 'modalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'colTitleText',
              component: 'ZdCol',
              align: 'center',
              cols: 12,
              children: [
                {
                  name: 'titleText',
                  component: 'ZdText',
                  text: 'Alertas',
                  cssClass: 'text-center',
                  style: {
                    color: '#0077B6',
                  },
                },
              ],
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              modalName: `alertModal-${idSensor}`,
            },
          ],
        },
        {
          name: 'form-alert',
          component: 'ZdForm',
          children: [
            {
              name: 'row2-1',
              component: 'ZdRow',
              children: [
                {
                  name: 'col2-1',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'typeAlert',
                      component: 'ZdSelect',
                      datasource: {
                        data: [],
                      },
                      dataValue: 'id',
                      dataText: 'name',
                      label: 'Tipo de alerta',
                      events: {
                        onCreated: this.listAlertTypes.bind(this),
                      },
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col-2-2',
                  component: 'ZdCol',
                  cols: 12,
                  children: [
                    {
                      name: 'contactAlert',
                      component: 'ZdTextInput',
                      label: 'Contato para o alerta',
                      validations: {
                        required: {},
                      },
                    },
                  ],
                },
                {
                  name: 'col3-1',
                  component: 'ZdCol',
                  cols: 12,
                  align: 'center',
                  children: [
                    {
                      name: `btn-${idSensor}`,
                      component: 'ZdButton',
                      label: 'Aplicar novo alerta',
                      events: {
                        click: this.createNewAlert.bind(this),
                      },
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
    const AlertModal: IModal = ModalService.create(aModal);
    this.mapAlertModal[idSensor] = AlertModal;
  }

  public createNewAlert({ component }: { component: Button }) {
    const formAlert: Form = Metadata.getInstance('form-alert');
    const validate = formAlert.validate();
    if (validate) {
      const splitedName = component.name.split('-');
      const idSensor = splitedName[1];
      const contactField: TextInput = Metadata.getInstance('contactAlert');
      const typeAlertField: Select = Metadata.getInstance('typeAlert');
      const bodyPost = {
        NRPARAMLEITCONTPESSOAS: parseInt(idSensor, 10),
        DSCONTATOSALERTA: contactField.value,
        IDTIPOALERTA: typeAlertField.value,
        NRORG: 20,
      };
      Http.post('/ParamAlertContPessoas', bodyPost)
        .then(() => {
          AlertService.show({
            name: 'basic-alert',
            text: `Novo alerta inserido com sucesso para o sensor ${idSensor}.`,
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
          typeAlertField.value = '';
        })
        .catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            text: err,
            top: true,
            left: false,
            timeout: 3000,
            color: 'error',
          });
        });
    }
  }

  public destroyAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].destroy();
  }

  public openAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].show();
  }

  public closeAlertModal({ component }: { component: Button }) {
    const macAddress = component.name;
    this.mapAlertModal[macAddress].hide();
  }

  public insertSensor() {
    const form: Form = Metadata.getInstance('sensorForm');
    const validate = form.validate();
    if (validate) {
      const bodyPost: IDictionary = {};
      let fields = this.getGeneralFields();
      const sensorFields = this.getSensorExclusiveFields();
      if (!this.disabledSensorFields) {
        fields = fields.concat(sensorFields);
      } else {
        sensorFields.forEach((item) => {
          const nameValue: string = item.name;
          bodyPost[nameValue] = 0;
        });
      }
      fields.forEach((field) => {
        const nameValue: string = field.name;
        bodyPost[nameValue] = Number.isNaN(field.value) ? field.value : parseFloat(field.value);
      });
      Http.post('/ParamLeitContPessoas', bodyPost)
        .then((response) => {
          const numInserted = response.data.dataset.ItemID[0];
          const row: Col = Metadata.getInstance('row2');
          const newCard = this.newCard(this.CardCounter.toString(), bodyPost.IDMACDEVICE, bodyPost.CDCLIENTE, numInserted.toString());
          row.children.push(newCard);
          AlertService.show({
            name: 'basic-alert',
            text: 'Novos parametros de sensor foram criados com sucesso.',
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
          fields.forEach((field) => {
            field.value = '';
          });
        }).catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            text: err,
            top: true,
            left: false,
            timeout: 3000,
            color: 'success',
          });
        });
    }
  }
}
