import {
  IDictionary, Http, Metadata,
  IEventParam, Loader, Router,
} from '@zeedhi/core';
import {
  Select, Container, Text, Card, Textarea, DialogService,
  Date, CheckboxMultiple, IComponentRender, Button,
  Form, TextInput, FileInput, AlertService,
} from '@zeedhi/common';
import {
  IterableCarousel, CardTitle,
} from '@zeedhi/teknisa-components-common';

import {
  getRouteParams,
} from '../router';

export class VistoriasController {
  public hierarquiasController = Loader.getInstance('HierarquiasController');

  public modalController = Loader.getInstance('NovoCriterioController');
  /* ---------------------- Tela Vistorias  --------------------- */

  // Variável que abriga o cards atividades
  public selected: IDictionary = [];

  public inspectionName: string = '';

  // Array com os valores de cada select
  public dataSelects: IDictionary = {
    inspectionLocalType: [],
    inspectionLocals: [],
    inspectionChecklists: [],
    inspectionEmployees: [],
  };

  // Recebe cada info do banco e retorna para o array cima
  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  // Inicializa as funções get
  public initializeData() {
    this.setLocalType();
    this.setLocals();
    this.setChecklists();
    this.setEmployees();
  }

  // Seta em cada select os valores do array dataSelects
  public setDataSelects({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  // Busca os "Tipos de Local"
  public setLocalType() {
    Http.get('/getLocalType').then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionLocalType.push(this.setData(database.NMTIPOLOCALCLI, database.NRTIPOLOCALCLI));
      });
    });
  }

  // Busca os "Locais de Atendimento"
  public setLocals() {
    Http.get('/getLocals').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.dataSelects.inspectionLocals.push(this.setData(database.NMLOCALCLIENTE, database.NRLOCALCLIENTE));
      });
    });
  }

  // Busca os "Checklists"
  public setChecklists() {
    Http.get('/getCheckListLocalAtend').then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionChecklists.push(this.setData(database.NMCHECKLIST, database.NRCHECKLIST));
      });
    });
  }

  // Busca Funcionários (Usando uma rota temporária, depois trocar para getUsers de UsuariosController)
  public async setEmployees() {
    const hierarchyUsersList: IDictionary = await this.hierarquiasController.mountHierarchyUsersArray();
    Http.get(`/getEmployee?params=${hierarchyUsersList}`).then((response: IDictionary) => {
      response.data.dataset.options.forEach((database: IDictionary) => {
        this.dataSelects.inspectionEmployees.push(this.setData(database.NMFUNCIONARIO, database.NRFUNCIONARIO));
      });
    });
  }

  // Monta e exibe os cards com as informações de cada vistoria
  public async mountInpectionsCards(selectsFilterInfo: IDictionary = {}) {
    const hierarchyUsersList: IDictionary = await this.hierarquiasController.mountHierarchyUsersArray();
    const containerObject = Metadata.getInstance<Container>('inpectionsListContainer');
    containerObject.children = [];

    Http.post('/getInspectionsInfo', { params: selectsFilterInfo, params2: hierarchyUsersList }).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        containerObject.children.push(< unknown > this.inspectionCard(database.NRGERENCIAMENTOCHK, database.NMGERENCIAMENTOCHK, database.NRCHECKLISTAGEND, database.NMLOCALCLIENTE, database.NMLOCALPAI, database.DTHORAAGEND, database.NMFUNCIONARIO, database.IDSITUACAOGERCHK, database.IDSITUACAOAGEND) as IComponentRender);
      });
    });
  }

  // Manuseia a data vinda do banco
  public timeManager(time: any) {
    const date = `${time[0].split('-')[2]}-${time[0].split('-')[1]}-${time[0].split('-')[0]} - ${time[1].substr(0, 5)}`;
    return date;
  }

  // Seta as cores de Situação
  public statusColor({ component }: IEventParam<Text>) {
    if (component.text === 'Aguardando Vistoria') {
      component.text = "<span  style='color: #ff6700; font-size: 14px;'>AGUARDANDO VISTORIA</span>";
    } else if (component.text === 'Em andamento') {
      component.text = "<span  style='color: #023e8a; font-size: 14px;'>EM ANDAMENTO</span>";
    } else if (component.text === 'Avaliado') {
      component.text = "<span  style='color: #2b9348; font-size: 14px;'>A V A L I A D O</span>";
    } else if (component.text === 'Tarefa Pendente') {
      component.text = "<span style='color: #ff6700; font-size: 14px;'>TAREFA PENDENTE</span>";
    } else {
      component.text = "<span style='color: #FA980D; font-size: 14px;'>CANCELADO</span>";
    }
  }

  // Início do eventro de filtro e exibição das vistorias
  public initializeFilter() {
    this.mountInpectionsCards(this.filter());
  }

  // Adiciona as informações dos selects em um array
  public filter() {
    const dateObject = Metadata.getInstance<Date>('inspectionDate').value;
    return {
      'LC.NRTIPOLOCALCLI': Metadata.getInstance<Select>('inspectionLocalType').value,
      'LC.NRLOCALCLIENTE': Metadata.getInstance<Select>('inspectionLocals').value,
      'GCHK.NRCHECKLIST': Metadata.getInstance<Select>('inspectionChecklists').value,
      'GCHK.NRFUNCIONARIO': Metadata.getInstance<Select>('inspectionEmployees').value,
      'GCHK.IDSITUACAOGERCHK': Metadata.getInstance<Select>('inspectionStatus').value,
      'TO_CHAR(GCHK.DTHORAAGEND,\'DD/MM/YYYY\')': ((dateObject != null) ? `TO_CHAR('${dateObject}')` : null),
    };
  }

  // Card de Vistorias
  public inspectionCard(nrgerenciamentochk: number, nmgerenciamentochk: string, nrchecklistagend: number, nmlocalcliente: string, nmlocalclientepai: string, dthoraagend: string, nmfuncionario: string, idsituacaogerchk: number, idsituacaoagend: number) {
    return {
      name: 'ColunaCard',
      component: 'ZdCol',
      cssClass: 'zd-px-0',
      children: [
        {
          name: `inspection${nrgerenciamentochk}`,
          component: 'ZdCard',
          to: `/vistorias/detalhes/${nrgerenciamentochk}`,
          events: {
            click: '{{VistoriasController.setInspectionToUpdate}}',
          },
          children: [
            {
              name: nmlocalcliente,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: nrchecklistagend,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: 'linhaInterna',
              component: 'ZdRow',
              justify: 'space-between',
              align: 'center',
              children: [
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: nmgerenciamentochk,
                      tag: 'h4',
                      style: {
                        color: '#2196f3',
                      },
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 3,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-map-marker-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: `${nmlocalcliente} | ${nmlocalclientepai}`,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-clock-check-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: dthoraagend,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '3',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-account-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: nmfuncionario,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 2,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaogerchk,
                          style: {
                            'font-size': '15px',
                          },
                          events: {
                            onCreated: '{{VistoriasController.statusColor}}',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaoagend,
                          isVisible: false,
                          style: {
                            'font-size': '15px',
                          },
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  /* ---------------------- Tela vistoriasDetalhes  --------------------- */

  public isVisibleElement: boolean = !(localStorage.getItem('idpermisssvistoria') === 'V');

  public isDisabledElement: boolean = (localStorage.getItem('idpermisssvistoria') === 'V');

  // Fecha a caixa de diálogo
  private dialogHideEvents: any = {
    click: () => {
      DialogService.hide();
    },
  };

  // Chama o evento de clear data e fecha a caixa de diálogo
  private dialogConfirmEvents: any = {
    click: () => {
      this.clearInspectionData();
      DialogService.hide();
    },
  };

  // Chama o evento de save inspection e fecha a caixa de diálogo
  private saveConfirmEvents: any = {
    click: () => {
      this.saveInspectionDetails(Metadata.getInstance<Button>('saveInspectionDt'));
      DialogService.hide();
    },
  };

  // Confirma que quer sair da página
  private finishConfirmEvents: any = {
    click: () => {
      this.finishInspectionDetails();
      DialogService.hide();
    },
  };

  // Recebe o nome e id da atividade
  public setActivityData(ActName: string, ActId: number) {
    return {
      id: ActId,
      name: ActName,
    };
  }

  // Recebe o nome, id e idmarcado do critério
  public setCriterioData(nm: string, id: number, markedId: string) {
    if (markedId === 'S') Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.push(id);
    return {
      name: nm,
      value: id,
      criterioId: markedId,
    };
  }

  public changeCardColor({ component }: IEventParam<Card>) {
    if (component.cssClass === '') {
      component.cssClass = 'inspectionCardClicked';
      component.children[0].color = '#ffffff';

      const id = component.name.substr(8);
      const subtitle = component.children[1].text;

      this.getCriteria(id, subtitle);
    } else { // Se tirar o clicar pela segunda vez tirar a seleção
      component.cssClass = '';
      component.children[0].color = '#0077b6';

      const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      checkObject!.datasource!.data = [];
    }
  }

  // Busca nome da Vistoria e Onde é feita
  public getInspectionNameAndLocal(nrgerenciamentochk: string) {
    Http.get(`/getInspectionInfo/${nrgerenciamentochk}/nameAndLocal`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('inspectionDtName').text = database.NMGERENCIAMENTOCHK;
        Metadata.getInstance<Text>('inspectionDtLocal').text = database.NMLOCALCLIENTE;
      });
    });
  }

  // Busca critérios
  public getCriteria(id:string, subtitle: string) {
    const params = id;
    const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const cardObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const btnObject = Metadata.getInstance<Button>('newCriterioinspectionDt');

    cardObject.subtitle = subtitle;
    checkObject!.datasource!.data = [];
    checkObject!.value = [];

    Http.get(`/getCriteria/${params}`).then((response: IDictionary) => {
      btnObject.disabled = (response.data.dataset.info[0].IDSITUACAOAGEND !== '4');
      response.data.dataset.info.forEach((database: IDictionary) => {
        checkObject?.datasource?.data?.push(this.setCriterioData(database.NMCRITERIOTARGERCHK, database.NRCRITERIOTARGERCHK, database.IDMARCADO));
      });
    });
  }

  // Busca as atividades
  public getActivities(id: string) {
    const cardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');

    Http.get(`/getActivities/${id}/Activities`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        cardObject.iterable.datasource.data.push(this.setActivityData(database.NMATIVIDADE, database.NRATIVIDADECHK));
      });
    });
  }

  // Busca critérios e observação
  public getNoteAndCriteria({ component }: IEventParam<Card>) {
    if (this.selected) {
      this.selected.cssClass = 'inspectionCard';
    }
    component.cssClass = 'inpectionCardClicked';
    this.selected = component;

    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');

    textAreaObject.disabled = this.isDisabledElement;
    fileInputObject.disabled = this.isDisabledElement;
    criteriaTitleObject.text = '- Marque apenas campos de não coformidade:';

    const params = {
      NRATIVIDADECHK: component.name.substr(8),
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
    };

    Http.post('/getInspectionToUpdate', { params }).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        if (database.IDSITUACAOAGEND !== '4') {
          this.noPermissonModify();// desativa o botão finalize, torna apenas visualização
        }
        Metadata.getInstance<Textarea>('inspectionDtNote').value = database.DSOBSTAREFAGERCHK;
        Metadata.getInstance<Text>('inspectionDtNrtarefagerchk').text = database.NRTAREFAGERCHK;
        Metadata.getInstance<Text>('inspectionDtNratividadechk').text = component.name.substr(8);
        this.getCriteria(database.NRTAREFAGERCHK, component.children[1].text);
        // Metadata.getInstance<FileInput>('inspectionDtImage').value = database.DSFOTOTAREFAAGEND;
        // Metadata.getInstance<Text>('inspectionDtName').text = component.children[0].name;
        // Metadata.getInstance<Text>('inspectionDtNR').text = database.NRGERENCIAMENTOCHK;
        // Metadata.getInstance<Text>('inspectionDtNM').text = database.NMGERENCIAMENTOCHK;
        // Metadata.getInstance<Text>('inspectionDtNRTAREFAGERCHK').text = database.NRTAREFAGERCHK;
      });
    });
  }

  // Mostra na tela as informações da vistoria
  public setInspectionToUpdate() {
    const routeParams = getRouteParams();
    const { id } = routeParams;

    Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text = id;

    this.getInspectionNameAndLocal(id);
    this.getActivities(id);
  }

  public noPermissonModify() {
    const chkbox = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const tArea = Metadata.getInstance<Textarea>('inspectionDtNote');
    const saveButton = Metadata.getInstance<Button>('saveInspectionDt');
    const newCriteria = Metadata.getInstance<Button>('newCriterioinspectionDt');
    const fileInput = Metadata.getInstance<FileInput>('inspectionDtImage');

    chkbox.readonly = true;
    tArea.readonly = true;
    saveButton.disabled = true;
    fileInput.disabled = true;
    newCriteria.disabled = true;
  }

  // Salva as informações de critérios e observação para a atividade selecionada
  public saveInspectionDetails({ component }: any) {
    let idSituation: any;
    let message: any;

    const params = {
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
      NRCRITERIOTARGERCHK: Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt')?.datasource?.data,
      NRCRITERIOTARGERCHKS: Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt').value,
      NRTAREFAGERCHK: Metadata.getInstance<Text>('inspectionDtNrtarefagerchk').text,
      DSOBSTAREFAGERCHK: Metadata.getInstance<Textarea>('inspectionDtNote').value,
      NRATIVIDADECHK: Metadata.getInstance<Text>('inspectionDtNratividadechk').text,
      // NRFOTOTAREFAGERCHK: Metadata.getInstance<FileInput>('inspectionDtImage').value,
    };

    if (component.name === 'saveInspectionDt') {
      idSituation = 2;
      message = 'Alterações salvas com sucesso!';
      Metadata.getInstance<Button>('FinalizeInspectionButton').disabled = false;
    } else {
      idSituation = 3;
      message = 'Vistoria realizada com sucesso!';
    }

    Http.post('/updateInspection', { idSituation, params }).then((response: IDictionary) => {
      const returnMessage = response.data.dataset.info[0];
      if (returnMessage !== 'OK') {
        DialogService.show({
          name: 'warning-dialog',
          type: 'warning',
          title: 'Erro!',
          text: 'Operação bloqueada! Ocorreu um erro ao salvar os dados desta vistoria.<br><br>Se o problema persistir entre em contato com o suporte.',
        });
      } else if (returnMessage === 'OK') {
        Metadata.getInstance<Button>('saveInspectionDt').disabled = true;
        Metadata.getInstance<Button>('cancelInspectionDt').disabled = true;

        AlertService.show({
          name: 'buttons-alert',
          text: message,
          color: 'success',
          top: true,
          left: false,
          timeout: 1500,
        });
      }
    });
  }

  // Ativa os botoões de salvar e cancelar
  public activateButtons() {
    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = false;
    cancelButtonObject.disabled = false;
  }

  // Limpa o dados da tela
  public clearInspectionData() {
    const chkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage');
    const cardTitleObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');
    const newCriterioObject = Metadata.getInstance<Button>('newCriterioinspectionDt');

    chkObject!.datasource!.data = [];
    textAreaObject.value = null;
    textAreaObject.disabled = true;
    fileInputObject.value = true;
    fileInputObject.disabled = true;
    cardTitleObject.subtitle = '';
    criteriaTitleObject.text = '- Nenhuma atividade selecionada.';
    newCriterioObject.disabled = true;

    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = true;
    cancelButtonObject.disabled = true;

    this.clearActivitiesColor();
  }

  // Tira a seleção dos cards de atividade
  public clearActivitiesColor() {
    const iterableCardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');
    iterableCardObject.children.forEach((activity: IDictionary) => {
      activity.color = '#ffffff';
    });
  }

  // Volta para tela de vistorias
  public finishInspectionDetails() {
    Router.replace('/vistorias');
  }

  // Confirma se quer cancelar
  public confirmCancelButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Cancelar',
      text: 'Deseja cancelar suas alterações?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.dialogConfirmEvents,
        },
      ],
    });
  }

  // Confirma se quer salvar
  public confirmSaveButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Salvar',
      text: 'Deseja salvar suas alterações para esta atividade?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.saveConfirmEvents,
        },
      ],
    });
  }

  // Confirma se quer deixar a página
  public confirmFinishButton() {
    DialogService.show({
      name: 'finishButton',
      type: 'warning',
      title: 'Concluir',
      text: 'Todas as alterações não salvas serão perdidas. Você deseja mesmo continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.finishConfirmEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
        },
      ],
    });
  }

  // Tratamento do modal para o botão "novo critério"
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    Promise.all([this.modalController.openBasicModal()]).then(() => {
      const buttonObject = Metadata.getInstance<Button>('closeModalButton');
      buttonObject.events = {
        click: () => this.newCriterio(),
      };
    });
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  // Adiciona um novo critério
  public newCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chkObject = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      const criterioName = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chkObject!.datasource!.data!.push({
        name: criterioName,
        value: null,
      });
      this.fecharModal();
    }
  }

  // Teste Adicionar foto
  public updloadPictures() {
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage').value;
    const arquivos: IDictionary = [];
    const arquivo = {
      FOTO: [],
    };
    const types: IDictionary = ['jpg', 'jpeg', 'png'];

    fileInputObject.forEach((database: IDictionary) => {
      const typeSplited = database.type.split('/', 2);
      const params = {
        name: database.name,
        type: typeSplited[1],
        size: database.size,
      };
      const verifica = types.filter((type:string) => type === params.type);
      if (verifica.length > 0) {
        arquivos.push(params);
      }
    });
    arquivo.FOTO = arquivos as any;
    // Http.post('/uploadPictures', { arquivo });
  }

  public consoleLog(variavel: any) {
    const titulo = Metadata.getInstance<Text>('titulo');
    titulo.text = variavel;
  }
}
